/*  simple_notification V. 1.0
    2024.06.27 Csercsa Soft

    Program célja: Egyszerű "notification" üzeneteg generálása, kezelése.
    
    Fájlok: 
        simple_notification.js, simple_notification.css, 
        media/warning.png, media/message.png, media/error.png

    Minden egyéb fájl csak a program működésének tesztelését, bemutatását szolgálja.

    Program használata:
        notifications.message(szöveg(kötelező), típus[null, warn, err], cím[szöveg], bezárás gomb[null, false] )
        notifications.messagehtml(szöveg / DOM objektum (kötelező), bezárás gomb[null, false] )
        notifications.clear() - elemek animált, szekvenciális törlése: Futás közben érkezett üzenetek nem fognak megjelenni!
        notifications.destroy() - Minden elem azonnali törlése.
*/
// -------------------------------------------------------------------------------------------------------------------------------
var notifications = (function(){
    // Alapbeállítások
    const SHOWED_CARD = 6000;                   // Kártyák megjelenítésének időtartama (ms)
    const MAX_SHOWED_CARD = 5;                  // Egyszerre megjeleníthető kártyák száma
    const REMOVE_ANIMATION_DELAY = 300;         // Eltüntetéskor ennyi idő áll a CSS animáció lefutására, mielőtt a HTML objektum törlődik (ms)
    const DEFAULT_NORMAL_TITLE = 'Message';     // Alapértelmezett kártya fejlécek, ha meghíváskor nincs megadva
    const DEFAULT_WARNING_TITLE = 'Warning!';
    const DEFAULT_ERROR_TITLE = 'Error!';
    // ---------------------------------------
    let parentHTMLObject = false;   // Az a HTML objektum, melyben megjeleni az üzenetmegjelenítés
    let frameHTMLObject = false;    // Szülőobjektum, mely az üzeneteket fogja tartalmazni
    let buffer = [];                // Ebben tároljuk majd az éppen meg nem jelenített üzeneteket
    let timer = false;              // Időzítő változó
    let clearBusy = false;

    // Ezt a funkciót híjuk meg, ha dinamikus, nem előre formázott üzenetet  küldünk
    // Igazából a "messageHandlert" is meghívhatnánk, de így egyszerűbb
    function htmlMessage(iHTML = false, iCloseButton = true){
        messageHandler(iHTML, 'HTML_DATA', null, iCloseButton);
    }

    // Üzenet fogadó, kezelő függvény
    function messageHandler(iText = false, iType = '', iTitle = '', iCloseButton = true){
        if(!iText || clearBusy){return}  // Ha nincs adat, vagy éppen a törlési szekvencia fut, akkor kilépünk a függvényből
        createFrame();      // Meghívjuk a funkciót, mely létrehozza a szülőobjektumot

        // Megvizsgáljuk, hogy a kártyák száma elérte e már a maximumot, melyet megadtunk
        // Ez a keret gyermekelmeinek számából tudjuk
        if(frameHTMLObject.children.length < MAX_SHOWED_CARD){
            // Ha még fér kártya, akkor meghívjuk a renderelő függvényt, mely objektumként megkapja az adatokat
            createCard({text:iText, type:iType, title:iTitle, clButton:iCloseButton});
        }else{
            // Ha "megtelt" a keret, az adatokat eltároljuk a bufferben, szintén objektumban
            buffer.push({text:iText, type:iType, title:iTitle, clButton:iCloseButton});
        }
        // Időzítő indítása...
        startTimer();
    }

    // Ezt a függvényt hívja meg az időzítő. Ez biztosítja az adatok megjelenítését, elemek folytonos vezérlését
    async function sequence(){
        // Mikor lejár az időzítő, mindig törlünk egy kártyát, majd visszakapjuk, hogy hány darab kártya maradt a keretben.
        let remainerCards = await removeCard();
        switch(true){
            // Ha van hely, valamint van adat a bufferben, akkor meghívjuk a renderelő függvényt
            case (remainerCards < MAX_SHOWED_CARD) && (buffer.length > 0):{
                createCard(buffer.shift());
                break;
            }
            // Ha nincs megjelenített kártya, valamint a buffer is üres, töröljük a keretelemet is
            case (remainerCards === 0) && (buffer.length === 0):{
                stopTimer();
                removeFrame();
            }
        }
    }

    // Kártya renderelő függvény
    function createCard(data){
        let newEl = document.createElement('DIV');
        newEl.classList.add('card_frame');          // Létrehozzuk a kártya keretét 
        
        // Megvizsgáljuk, hogy milyen típusú adat érkezett
        // A "HTML_DATA" típust eredetileg akkor kapjuk, ha "htmlMessage()" fügvénnyel küldünk adatot, azonban ez a sima "messageHandler()" függvénnyel is elérhető
        // Itt szűrjük ki, hogy objektum, vagy string formában érkezett az adat
        if(data.type === 'HTML_DATA'){
            switch(true){
                case typeof(data.text) === 'string':{
                    newEl.innerHTML = data.text;
                    break;
                }
                case (typeof(data.text) === 'object' && data.text.nodeType !== undefined):{
                    newEl.appendChild(data.text);
                    break;
                }
            }
        }else{
            // Ha nem "HTML_DATA" típus, akkor előformázott kártyát renderelünk
            createPreFormattedCard();
        }
        // Megnézzük kell e törlés gomb alapértelmezetten igen)
        if(data.clButton){
            newEl.appendChild(createCloseButton());
        }
        // Hozzáadjuk a létrehozott kártyát a keretobjektumhoz
        frameHTMLObject.appendChild(newEl);

        // Előre formázott kártya generálása
        function createPreFormattedCard(){
            let elClassName;
            // Megvizsgáljuk, milyen típusú adat érkezett, és azt, van e cím
            switch(data.type){
                case 'warn':{
                    elClassName = 'warning';
                    if(!data.title){
                        data.title = DEFAULT_WARNING_TITLE;
                    }
                    break; 
                }
                case 'err':{
                    elClassName = 'error';
                    if(!data.title){
                        data.title = DEFAULT_ERROR_TITLE;
                    }
                    break; 
                }
                default:{
                    elClassName = 'normal';
                    if(!data.title){
                        data.title = DEFAULT_NORMAL_TITLE;
                    }
                }
            }
            // Kártya HTML forrása, hozzáadva a CSS típust
            newEl.innerHTML = `
            <h3 class="${elClassName}">${data.title}</h3><div class="card_content card_content-${elClassName}">
            <div class="card_icon_box card_icon_box-${elClassName}"></div><p class="${elClassName}">${data.text}</p>
            </div>
            `;
        }
    }

    // Első kártyak törlése
    async function removeCard(){
        if(frameHTMLObject.children.length > 0){
            frameHTMLObject.firstElementChild.classList.add('card_frame-unset');
            await new Promise(resolve => setTimeout(resolve, REMOVE_ANIMATION_DELAY));
            frameHTMLObject.firstElementChild.remove();
        }
        // A funkció visszaadja, hogy hány kártya maradt még
        return frameHTMLObject.children.length
    }

    // Ez a függvény azt az elemet törli, melyen megnyomtuk a törlés gombot
    async function removeThisCard(e){
        e.target.parentElement.classList.add('card_frame-unset');
        await new Promise(resolve => setTimeout(resolve, REMOVE_ANIMATION_DELAY));
        e.target.parentElement.remove();
        //Ha a buffer nem üres, kivesszük a soron következő üzenetet
        if(buffer.length > 0){
            createCard(buffer.shift());
        }
    }

    // Keret renderelése
    function createFrame(){
        if(frameHTMLObject){return} // Ha már létezik, nem hozunk létre újat
        frameHTMLObject = document.createElement('DIV');
        frameHTMLObject.classList.add('simple_notification');
        try{
            frameHTMLObject.addEventListener('touchstart', mouseOver);
            frameHTMLObject.addEventListener('touchend', mouseOut);
            frameHTMLObject.addEventListener('touchcancel', mouseOut);
        }catch(e){}

        frameHTMLObject.addEventListener('mouseover', mouseOver);
        frameHTMLObject.addEventListener('mouseout', mouseOut);
        if(!parentHTMLObject){
            parentHTMLObject = document.body;
        }
        parentHTMLObject.appendChild(frameHTMLObject);
    }

    // Keret törlése, ha elfogytak az üzenetek
    function removeFrame(){
        if(!frameHTMLObject){return}
        frameHTMLObject.remove();
        frameHTMLObject = false;
    }

    // Bezárás gomb renderelése
    function createCloseButton(){
        let newButton = document.createElement('DIV');
        newButton.classList.add('card_close_button');
        newButton.addEventListener('click', removeThisCard);
        newButton.innerHTML = 'X';
        newButton.title = "Üzenet törlése";
        return newButton
    }

    // Üzenetek animált (szekvenciális) törlése
    async function clearMessages(){
        if(!frameHTMLObject || clearBusy){return}
        clearBusy = true;
        stopTimer();
        buffer = [];
        let cards = await removeCard();
        while(cards > 0){
            cards = await removeCard();
        }
        clearBusy = false;
        removeFrame();
    }

    // A teljes objektum azonnali megsemmisítése
    function destroyMessages(){
        if(!frameHTMLObject || clearBusy){return}
        stopTimer();
        buffer = [];
        removeFrame();
    }

    // Időzítő valamint esemény vezérlők
    function startTimer(){
        if(!timer){
            timer = setInterval(sequence, SHOWED_CARD);
        }
    }

    function stopTimer(){
        if(timer){
            clearInterval(timer);
            timer = false;
        }
    }
    
    function mouseOver(){
        stopTimer();
    }

    function mouseOut(){
        startTimer();
    }

    return {message:messageHandler, messagehtml:htmlMessage, clear:clearMessages, destroy:destroyMessages}
})();